<?
/**
 * @file
 *
 * Template System - controls the output of Drupal.
 *
 * The template system allows for the output of the Drupal system to be
 * customized by templates using any of the available template engines
 *
 */

/**
 * Initializes the template engine specified by the first argument. 
 */
function engine_init($eng) {
  global $theme, $engine;
  $engine = $eng;

  $engine_file = "engines/$engine/$engine.engine";

  include_once($engine_file);
  call_user_func("engine_$engine" . "_init");
}

/**
 * Execute a template engine call
 */
function engine_callback($hook, $options) {
  if ($func = engine_get_callback($hook)) {
    return call_user_func_array($func, $options);
  }
  elseif ($func = engine_get_callback("_default")) {
    return call_user_func_array($func, $options);
  }
}

/**
 * Define a template engine callback, to be executed for
 * the hook specified. 
 *
 * A default set of callbacks are already specified. 
 */
function engine_set_callback($hook, $function = null) {
  static $callback = array(
    "page"    => "engine_page",
    "node"    => "engine_node",
    "block"   => "engine_block",
    "box"     => "engine_box",
    "comment" => "engine_comment"
  );
 
  if ($function == null) {  
    return $callback[$hook];
  } 
  else {
    $callback[$hook] = $function;
  }

}

/**
 * Retrieve a callback for a certain template hook
 */
function engine_get_callback($hook) {
  return engine_set_callback($hook);
}

/**
 * Prepare the values passed to the theme_page function to be passed
 * into a pluggable template engine. 
 */
function engine_page($content, $title = NULL, $breadcrumb = NULL) {
  global $engine, $theme;

  /* Set title and breadcrumb to declared values */
  if (isset($title)) {
    drupal_set_title($title);
  }
  if (isset($breadcrumb)) {
    drupal_set_breadcrumb($breadcrumb);
  }
  
  $messages = array();
  if ($msg = drupal_get_messages()) {
    foreach ($msg as $message) {
      list($message, $type) = $message;
      $messages[] = array(
        "message" =>  ucfirst($message),
        "type"    => $type
      );
    }
  }


  if ($_GET['q'] == variable_get('site_frontpage', 'node')) {
    $mission = $theme['settings']['mission'] ? $theme['settings']['mission'] : variable_get('site_mission', "");
    $frontpage = true;
  }
 
  if ($theme['settings']['toggle_logo']) {
    if ($theme['settings']['default_logo']) {
      $logo = $theme['style_dir'] . '/logo.png'; 
    }
    elseif ($theme['settings']['logo_path']) {
      $logo = $theme['settings']['logo_path'];
    }
  }
  
  if ($theme['settings']['toggle_primary_links']) {
    if ($theme['settings']['primary_links']) {
      $primary_links = $theme['settings']['primary_links'];
    }
    else {
      $primary_links = link_page();
    }
  }
  if ($theme['settings']['toggle_secondary_links']) {
    $secondary_links =  $theme['settings']['secondary_links'];
  }
 
  $vars = array(
    "head_title"          => (drupal_get_title() ? drupal_get_title() ." | ". variable_get("site_name", "drupal") : variable_get("site_name", "drupal") ." | ". variable_get("site_slogan", "")),
    "site"                => variable_get("site_name", "drupal"),
    "head"                => drupal_get_html_head(),
    "onload_attributes"   => theme_onload_attribute(),
    "directory"           => $theme["dirname"],
    "logo"                => $logo,
    "site_name"           => ($theme['settings']['toggle_name'] ? variable_get('site_name', '') : ''),
    "site_slogan"         => ($theme['settings']['toggle_slogan'] ? variable_get('site_slogan', '') : ''),
    "title"               => drupal_get_title(), 
    "primary_links"       => is_array($primary_links) ?  theme("links", $primary_links) : $primary_links,
    "secondary_links"     => is_array($secondary_links) ?  theme("links", $secondary_links) : $secondary_links,
    "breadcrumb"          => theme("breadcrumb", drupal_get_breadcrumb()),
    "tabs"                => theme('menu_local_tasks'),
    "messages"            => $messages,
    "help"                => menu_get_active_help(),
"stylesheet"          => $theme['style_file'],
    "mission"             => $mission,
    "is_front"            => $frontpage,
    "sidebar_left"        => $blocks = theme("blocks", "left"),
    "content"             => "\n<!-- begin content -->\n$content<!-- end content -->\n",
    "sidebar_right"       => theme("blocks", "right"),
    "footer_message"      => variable_get("site_footer", FALSE),
    "closure"             => theme("closure")
  );
  return call_user_func("engine_$engine" . "_page", $vars);
}

/**
 * Prepare the values passed to the theme_node function to be passed
 * into a pluggable template engine. 
 */
function engine_node($node, $main = 0, $page = 0) {
  global $engine, $theme;
  $taxonomy = taxonomy_link("taxonomy terms", $node);
  $links = link_node($node, $main);

  $vars = array(
      "title"          => $node->title,
      "node_url"       => url("node/view/$node->nid"),
      "terms"          => $terms,
      "name"           => format_name($node),
      "avatar"         => $avatar,
      "date"           => format_date($node->created),
      "static"         => $node->static,
      "picture"        => theme('user_picture', $node),
      "content"        => ($main && $node->teaser) ? $node->teaser : $node->body,
      "links"          => ($links != "") ? theme("links", $links) : "",
      "mission"        => $mission,
      "page"           => $page,
      "taxonomy"       => $taxonomy,

    /* Lastly , pass the actual node to allow more customization */
      "node"           => $node,
      "main"           => $main,
      "page"           => $page
    );

  // Display info only on certain node types. 
  if ($theme['settings']['toggle_node_info_' . $node->type]) {
    $vars['submitted'] =  t("Submitted by %a on %b.", array("%a" => format_name($node), "%b" => format_date($node->created)));
  }

  return call_user_func("engine_$engine" . "_node", $vars);
}

/**
 * Prepare the values passed to the theme_comment function to be passed
 * into a pluggable template engine. 
 */
function engine_comment($comment, $link = 0) {
  global $engine;
  return call_user_func("engine_$engine" . "_comment", array(
    "new"       => t("new"),
    "comment" => $comment,
    "submitted" => t("Submitted by %a on %b.",
                      array("%a" => format_name($comment),
                            "%b" => format_date($comment->timestamp))),
    "title"     => l($comment->subject, $_GET['q'], NULL, NULL, "comment-$comment->cid"),

    "picture" => theme('user_picture', $comment),
    "link"    => $link,
    "author"  => format_name($comment),
    "date"    => format_date($comment->timestamp))
  );
}

/**
 * Prepare the values passed to the theme_block function to be passed
 * into a pluggable template engine. 
 */
function engine_block($block) {
  global $engine;
  return call_user_func("engine_$engine" . "_block", array(
    "block" => $block
  ));
}

/**
 * Prepare the values passed to the theme_box function to be passed
 * into a pluggable template engine. 
 */
function engine_box($title, $content, $region = "main") {
  global $engine;
  return call_user_func("engine_$engine" . "_box", array(
    "title"   =>   $title,
    "content" =>   $content,
    "region"  =>   $region
  ));
}


?>
